@@ -21,6 +21,8 @@
  * Copyright (C) 2001-2012 Alexandre Cassen, <acassen@gmail.com>
  */
 
+#include "config.h"
+
 #include <openssl/err.h>
 #include "check_http.h"
 #include "check_ssl.h"
@@ -30,7 +32,7 @@
 #include "parser.h"
 #include "utils.h"
 #include "html.h"
-#ifndef _HAVE_SOCK_CLOEXEC_
+#if !HAVE_DECL_SOCK_CLOEXEC
 #include "old_socket.h"
 #include "string.h"
 #endif
@@ -774,7 +776,7 @@ http_connect_thread(thread_t * thread)
 		return 0;
 	}
 
-#ifndef _HAVE_SOCK_CLOEXEC_
+#if !HAVE_DECL_SOCK_CLOEXEC
 	if (set_sock_flags(fd, F_SETFD, FD_CLOEXEC))
 		log_message(LOG_INFO, "Unable to set CLOEXEC on http_connect socket - %s (%d)", strerror(errno), errno);
 #endif
