@@ -42,7 +42,6 @@ typedef struct _checker {
 	void				(*free_func) (void *);
 	void				(*dump_func) (void *);
 	int				(*launch) (struct _thread *);
-	int				(*plugin_launch) (void *);
 	virtual_server_t		*vs;	/* pointer to the checker thread virtualserver */
 	real_server_t			*rs;	/* pointer to the checker thread realserver */
 	void				*data;
@@ -74,7 +73,7 @@ extern list checkers_queue;
 
 /* Prototypes definition */
 extern void init_checkers_queue(void);
-extern void dump_conn_opts (conn_opts_t *);
+extern void dump_conn_opts(void *);
 extern void queue_checker(void (*free_func) (void *), void (*dump_func) (void *)
 			  , int (*launch) (thread_t *)
 			  , void *
@@ -86,7 +85,5 @@ extern void install_checkers_keyword(void);
 extern void install_connect_keywords(void);
 extern void warmup_handler(vector_t *);
 extern void update_checker_activity(sa_family_t, void *, int);
-extern void checker_set_dst(struct sockaddr_storage *);
-extern void checker_set_dst_port(struct sockaddr_storage *, uint16_t);
 
 #endif
