@@ -33,24 +33,17 @@
 #include <stdbool.h>
 
 #include <net/if.h>
-#include <netinet/ip_icmp.h>
+//#include <netinet/ip_icmp.h>
 #include <netinet/udp.h>
 #include <netinet/tcp.h>
 #include <sys/wait.h>
 
 #ifdef _WITH_LVS_
-#ifdef _KRNL_2_4_
-  #include "../libipvs-2.4/libipvs.h"
-  #include <net/ip_vs.h>
-#elif _KRNL_2_6_
-  #include "../libipvs-2.6/ip_vs.h"
-  #include "../libipvs-2.6/libipvs.h"
+  #include "libipvs.h"
+  #include "ip_vs.h"
+#ifdef _WITH_LVS_
+  #include "vrrp.h"
 #endif
-//  #include <net/ip_vs.h>
-#endif
-
-#ifndef IP_VS_TEMPLATE_TIMEOUT
-#define IP_VS_TEMPLATE_TIMEOUT IPVS_SVC_PERSISTENT_TIMEOUT
 #endif
 
 /* locale includes */
@@ -61,7 +54,7 @@
 #define IPVS_SUCCESS	1
 #define IPVS_CMD_DELAY	3
 
-#ifdef _HAVE_IPVS_SYNCD_
+#ifdef _WITH_LVS_
 #define IPVS_STARTDAEMON	IP_VS_SO_SET_STARTDAEMON
 #define IPVS_STOPDAEMON		IP_VS_SO_SET_STOPDAEMON
 #define IPVS_FLUSH		IP_VS_SO_SET_FLUSH
@@ -91,22 +84,38 @@ do {						\
 		UNSET_ALIVE((V));		\
 } while (0)
 
+#ifdef _WITH_LVS_
+struct lvs_syncd_config {
+	char				*ifname;	/* handle LVS sync daemon state using this */
+	vrrp_t				*vrrp;		/* instance FSM & running on specific interface */
+	int				syncid;		/* 0 .. 255 */
+#ifdef _HAVE_IPVS_SYNCD_ATTRIBUTES_
+	int				sync_maxlen;
+	struct sockaddr_storage		mcast_group;
+	int				mcast_port;
+	int				mcast_ttl;
+#endif
+	char				*vrrp_name;	/* used during configuration and SNMP */
+};
+#endif
+
 /* prototypes */
 extern int ipvs_start(void);
 extern void ipvs_stop(void);
+extern void ipvs_set_timeouts(int, int, int);
 extern void ipvs_flush_cmd(void);
 extern virtual_server_group_t *ipvs_get_group_by_name(char *, list);
 extern void ipvs_group_sync_entry(virtual_server_t *vs, virtual_server_group_entry_t *vsge);
 extern void ipvs_group_remove_entry(virtual_server_t *, virtual_server_group_entry_t *);
 extern int ipvs_cmd(int, virtual_server_t *, real_server_t *);
-extern void ipvs_syncd_cmd(int, char *, int, int, bool);
-extern void ipvs_syncd_master(char *, int);
-extern void ipvs_syncd_backup(char *, int);
+#ifdef _WITH_LVS_
+extern void ipvs_syncd_cmd(int, const struct lvs_syncd_config *, int, bool, bool);
+extern void ipvs_syncd_master(const struct lvs_syncd_config *);
+extern void ipvs_syncd_backup(const struct lvs_syncd_config *);
+#endif
 
-#ifdef _KRNL_2_6_
 /* Refresh statistics at most every 5 seconds */
 #define STATS_REFRESH 5
 extern void ipvs_update_stats(virtual_server_t * vs);
-#endif
 
 #endif
