@@ -35,7 +35,6 @@
 
 /* global defs */
 #define SMTP_PORT_STR		"25"
-#define SMTP_PORT		25
 #define SMTP_BUFFER_LENGTH	512
 #define SMTP_BUFFER_MAX		1024
 #define SMTP_MAX_FSM_STATE	10
@@ -68,15 +67,16 @@ do {					\
 } while (0)
 
 /* SMTP thread arguments */
-typedef struct _smtp_thread_arg {
-	int fd;
-	int stage;
-	int email_it;
-	char *subject;
-	char *body;
-	char *buffer;
-	long buflen;
-} smtp_thread_arg;
+typedef struct _smtp {
+	int		fd;
+	int		stage;
+	int		email_it;
+	char		*subject;
+	char		*body;
+	char		*buffer;
+	char		*email_to;
+	long		buflen;
+} smtp_t;
 
 /* SMTP command string processing */
 #define SMTP_HELO_CMD    "HELO %s\r\n"
@@ -84,12 +84,14 @@ typedef struct _smtp_thread_arg {
 #define SMTP_RCPT_CMD    "RCPT TO:<%s>\r\n"
 #define SMTP_DATA_CMD    "DATA\r\n"
 #define SMTP_HEADERS_CMD "Date: %s\r\nFrom: %s\r\nSubject: %s\r\n" \
-			 "X-Mailer: Keepalived\r\n\r\n"
+			 "X-Mailer: Keepalived\r\nTo: %s\r\n\r\n"
 #define SMTP_BODY_CMD    "%s\r\n"
 #define SMTP_SEND_CMD    "\r\n.\r\n"
 #define SMTP_QUIT_CMD    "QUIT\r\n"
 
+#define FMT_SMTP_HOST()	inet_sockaddrtopair(&global_data->smtp_server)
+
 /* Prototypes defs */
-extern void smtp_alert(real_server *, vrrp_rt *, vrrp_sgroup *,
+extern void smtp_alert(real_server_t *, vrrp_t *, vrrp_sgroup_t *,
 		       const char *, const char *);
 #endif
