@@ -56,25 +56,28 @@ typedef struct _sock {
 typedef struct _vrrp_data {
 	list			static_addresses;
 	list			static_routes;
+	list			static_rules;
 	list			vrrp_sync_group;
 	list			vrrp;
 	list			vrrp_index;
 	list			vrrp_index_fd;
 	list			vrrp_socket_pool;
 	list			vrrp_script;
+	list			vrrp_switch;
 } vrrp_data_t;
 
 /* Global Vars exported */
 extern vrrp_data_t *vrrp_data;
 extern vrrp_data_t *old_vrrp_data;
 extern char *vrrp_buffer;
+extern size_t vrrp_buffer_len;
 
 /* prototypes */
 extern void alloc_saddress(vector_t *);
 extern void alloc_sroute(vector_t *);
+extern void alloc_srule(vector_t *);
 extern void alloc_vrrp_sync_group(char *);
 extern void alloc_vrrp(char *);
-extern vrrp_stats *alloc_vrrp_stats(void);
 extern void alloc_vrrp_unicast_peer(vector_t *);
 extern void alloc_vrrp_track(vector_t *);
 extern void alloc_vrrp_script(char *);
@@ -82,7 +85,8 @@ extern void alloc_vrrp_track_script(vector_t *);
 extern void alloc_vrrp_vip(vector_t *);
 extern void alloc_vrrp_evip(vector_t *);
 extern void alloc_vrrp_vroute(vector_t *);
-extern void alloc_vrrp_buffer(void);
+extern void alloc_vrrp_vrule(vector_t *);
+extern void alloc_vrrp_buffer(size_t);
 extern void free_vrrp_buffer(void);
 extern vrrp_data_t *alloc_vrrp_data(void);
 extern void free_vrrp_data(vrrp_data_t *);
