@@ -11,7 +11,7 @@
  *              but WITHOUT ANY WARRANTY; without even the implied warranty of
  *              MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  *              See the GNU General Public License for more details.
- *              
+ *
  *              This program is free software; you can redistribute it and/or
  *              modify it under the terms of the GNU General Public License
  *              as published by the Free Software Foundation; either version
@@ -26,19 +26,22 @@
 /* system includes */
 #include <linux/icmpv6.h>
 
+/* local includes */
+#include "vrrp.h"
+
 /* local definitions */
-#define ETHERNET_HW_LEN 6
+#define ETHERNET_HW_LEN	6
 #define NEXTHDR_ICMP	58
 #define NDISC_HOPLIMIT	255
 
 /*
  *	ICMPv6 codes for Neighbour Discovery messages
  */
-#define NDISC_ROUTER_SOLICITATION       133
-#define NDISC_ROUTER_ADVERTISEMENT      134
-#define NDISC_NEIGHBOUR_SOLICITATION    135
-#define NDISC_NEIGHBOUR_ADVERTISEMENT   136
-#define NDISC_REDIRECT                  137
+#define NDISC_ROUTER_SOLICITATION	133
+#define NDISC_ROUTER_ADVERTISEMENT	134
+#define NDISC_NEIGHBOUR_SOLICITATION	135
+#define NDISC_NEIGHBOUR_ADVERTISEMENT	136
+#define NDISC_REDIRECT			137
 
 /*
  *	Neighbour Discovery option codes
@@ -69,7 +72,7 @@ struct ip6hdr {
 };
 
 /*
- *	NDISC Neighbour Advertisement related	
+ *	NDISC Neighbour Advertisement related
  */
 struct ndhdr {
 	struct icmp6hdr		icmph;
@@ -86,7 +89,8 @@ struct nd_opt_hdr {
 /* prototypes */
 extern void ndisc_init(void);
 extern void ndisc_close(void);
-extern int ndisc_send_unsolicited_na(ip_address *);
+extern void ndisc_send_unsolicited_na(vrrp_t *, ip_address_t *);
+extern int ndisc_send_unsolicited_na_immediate(interface_t *, ip_address_t *);
 
 #endif
 
