@@ -20,6 +20,8 @@
  * Copyright (C) 2001-2012 Alexandre Cassen, <acassen@gmail.com>
  */
 
+#include "config.h"
+
 /* system includes */
 #include <unistd.h>
 #include <net/ethernet.h>
@@ -33,7 +35,7 @@
 #include "vrrp_if_config.h"
 #include "vrrp_scheduler.h"
 #include "vrrp_ndisc.h"
-#ifndef _HAVE_SOCK_CLOEXEC_
+#if !HAVE_DECL_SOCK_CLOEXEC
 #include "old_socket.h"
 #endif
 #include "bitops.h"
@@ -144,9 +146,6 @@ ndisc_send_unsolicited_na_immediate(interface_t *ifp, ip_address_t *ipaddress)
 	char *nd_opt_lladdr = (char *) ((char *)nd_opt_h + sizeof(struct nd_opt_hdr));
 	char *lladdr = (char *) IF_HWADDR(ipaddress->ifp);
 	int len;
-	bool router;
-
-	router = get_ipv6_forwarding(ifp);
 
 	/* Ethernet header:
 	 * Destination ethernet address MUST use specific address Mapping
@@ -169,7 +168,7 @@ ndisc_send_unsolicited_na_immediate(interface_t *ifp, ip_address_t *ipaddress)
 
 	/* ICMPv6 Header */
 	icmp6h->icmp6_type = NDISC_NEIGHBOUR_ADVERTISEMENT;
-	icmp6h->icmp6_router = router;
+	icmp6h->icmp6_router = ifp->gna_router;
 
 	/* Override flag is set to indicate that the advertisement
 	 * should override an existing cache entry and update the
@@ -250,7 +249,7 @@ ndisc_init(void)
 
 	/* Create the socket descriptor */
 	ndisc_fd = socket(PF_PACKET, SOCK_RAW | SOCK_CLOEXEC, htons(ETH_P_IPV6));
-#ifndef _HAVE_SOCK_CLOEXEC_
+#if !HAVE_DECL_SOCK_CLOEXEC
 	set_sock_flags(ndisc_fd, F_SETFD, FD_CLOEXEC);
 #endif
 }
