@@ -23,8 +23,10 @@
 #ifndef _BITOPS_H
 #define _BITOPS_H
 
+#include <limits.h>
+
 /* Defines */
-#define BIT_PER_LONG	32
+#define BIT_PER_LONG	(CHAR_BIT * sizeof(unsigned long))
 #define BIT_MASK(idx)	(1UL << ((idx) % BIT_PER_LONG))
 #define BIT_WORD(idx)	((idx) / BIT_PER_LONG)
 
@@ -64,6 +66,9 @@ enum global_bits {
 	DONT_RESPAWN_BIT = 6,
 	RELEASE_VIPS_BIT = 7,
 	MEM_ERR_DETECT_BIT = 8,
+#ifdef _MEM_CHECK_LOG_
+	MEM_CHECK_LOG_BIT = 9,
+#endif
 };
 
 #endif
