@@ -22,11 +22,14 @@
  * Copyright (C) 2001-2012 Alexandre Cassen, <acassen@linux-vs.org>
  */
 
+#include "config.h"
+
 #ifdef _MEM_CHECK_
 #include <assert.h>
 #include <unistd.h>
 #include <fcntl.h>
 #include <stdio.h>
+#include <stdlib.h>
 #endif
 
 #include <errno.h>
@@ -41,6 +44,8 @@
 /* Global var */
 size_t mem_allocated;		/* Total memory used in Bytes */
 size_t max_mem_allocated;	/* Maximum memory used in Bytes */
+
+const char *terminate_banner;	/* banner string for report file */
 #endif
 
 static void *
@@ -103,7 +108,7 @@ typedef struct {
 	char *func;
 	char *file;
 	void *ptr;
-	unsigned long size;
+	size_t size;
 	long csum;
 } MEMCHECK;
 
@@ -118,7 +123,7 @@ static int f = 0;		/* Free list pointer */
 static FILE *log_op = NULL;
 
 void *
-keepalived_malloc(unsigned long size, char *file, char *function, int line)
+keepalived_malloc(size_t size, char *file, char *function, int line)
 {
 	void *buf;
 	int i = 0;
@@ -148,11 +153,11 @@ keepalived_malloc(unsigned long size, char *file, char *function, int line)
 	alloc_list[i].csum = check;
 	alloc_list[i].type = 9;
 
-	fprintf(log_op, "zalloc[%3d:%3d], %p, %4ld at %s, %3d, %s\n",
+	fprintf(log_op, "zalloc[%3d:%3d], %p, %4zu at %s, %3d, %s\n",
 	       i, number_alloc_list, buf, size, file, line, function);
 #ifdef _MEM_CHECK_LOG_
 	if (__test_bit(MEM_CHECK_LOG_BIT, &debug))
-		log_message(LOG_INFO, "zalloc[%3d:%3d], %p, %4ld at %s, %3d, %s\n",
+		log_message(LOG_INFO, "zalloc[%3d:%3d], %p, %4zu at %s, %3d, %s\n",
 		       i, number_alloc_list, buf, size, file, line, function);
 #endif
 
@@ -193,7 +198,7 @@ keepalived_free(void *buffer, char *file, char *function, int line)
 				mem_allocated -= alloc_list[i].size - sizeof(long);
 			} else {
 				alloc_list[i].type = 1;	/* Overrun */
-				fprintf(log_op, "free corrupt, buffer overrun [%3d:%3d], %p, %4ld at %s, %3d, %s\n",
+				fprintf(log_op, "free corrupt, buffer overrun [%3d:%3d], %p, %4zu at %s, %3d, %s\n",
 				       i, number_alloc_list,
 				       buf, alloc_list[i].size, file,
 				       line, function);
@@ -228,19 +233,19 @@ keepalived_free(void *buffer, char *file, char *function, int line)
 		return n;
 	}
 
-	if (buffer != NULL)
-		free(buffer);
-
-	fprintf(log_op, "free  [%3d:%3d], %p, %4ld at %s, %3d, %s\n",
+	fprintf(log_op, "free  [%3d:%3d], %p, %4zu at %s, %3d, %s\n",
 	       i, number_alloc_list, buf,
 	       alloc_list[i].size, file, line, function);
 #ifdef _MEM_CHECK_LOG_
 	if (__test_bit(MEM_CHECK_LOG_BIT, &debug))
-		log_message(LOG_INFO, "free  [%3d:%3d], %p, %4ld at %s, %3d, %s\n",
+		log_message(LOG_INFO, "free  [%3d:%3d], %p, %4zu at %s, %3d, %s\n",
 		       i, number_alloc_list, buf,
 		       alloc_list[i].size, file, line, function);
 #endif
 
+	if (buffer != NULL)
+		free(buffer);
+
 	free_list[f].file = file;
 	free_list[f].line = line;
 	free_list[f].func = function;
@@ -255,21 +260,21 @@ keepalived_free(void *buffer, char *file, char *function, int line)
 	return n;
 }
 
-void
-keepalived_free_final(char *banner)
+static void
+keepalived_free_final(void)
 {
 	unsigned int sum = 0, overrun = 0, badptr = 0;
 	int i, j;
 	i = 0;
 
-	fprintf(log_op, "\n---[ Keepalived memory dump for (%s)]---\n\n", banner);
+	fprintf(log_op, "\n---[ Keepalived memory dump for (%s)]---\n\n", terminate_banner);
 
 	while (i < number_alloc_list) {
 		switch (alloc_list[i].type) {
 		case 3:
 			badptr++;
 			fprintf
-			    (log_op, "null pointer to realloc(nil,%ld)! at %s, %3d, %s\n",
+			    (log_op, "null pointer to realloc(nil,%zu)! at %s, %3d, %s\n",
 			     alloc_list[i].size, alloc_list[i].file,
 			     alloc_list[i].line, alloc_list[i].func);
 			break;
@@ -284,7 +289,7 @@ keepalived_free_final(char *banner)
 				if (free_list[j].ptr == alloc_list[i].ptr)
 					if (free_list[j].type == 8)
 						fprintf
-						    (log_op, "  -> pointer allready released at [%3d:%3d], at %s, %3d, %s\n",
+						    (log_op, "  -> pointer already released at [%3d:%3d], at %s, %3d, %s\n",
 						     (int) free_list[j].csum,
 						     number_alloc_list,
 						     free_list[j].file,
@@ -299,7 +304,7 @@ keepalived_free_final(char *banner)
 			break;
 		case 1:
 			overrun++;
-			fprintf(log_op, "%p [%3d:%3d], %4ld buffer overrun!:\n",
+			fprintf(log_op, "%p [%3d:%3d], %4zu buffer overrun!:\n",
 			       alloc_list[i].ptr, i, number_alloc_list,
 			       alloc_list[i].size);
 			fprintf(log_op, " --> source of malloc: %s, %3d, %s\n",
@@ -308,7 +313,7 @@ keepalived_free_final(char *banner)
 			break;
 		case 9:
 			sum += alloc_list[i].size;
-			fprintf(log_op, "%p [%3d:%3d], %4ld not released!:\n",
+			fprintf(log_op, "%p [%3d:%3d], %4zu not released!:\n",
 			       alloc_list[i].ptr, i, number_alloc_list,
 			       alloc_list[i].size);
 			fprintf(log_op, " --> source of malloc: %s, %3d, %s\n",
@@ -319,7 +324,7 @@ keepalived_free_final(char *banner)
 		i++;
 	}
 
-	fprintf(log_op, "\n\n---[ Keepalived memory dump summary for (%s) ]---\n", banner);
+	fprintf(log_op, "\n\n---[ Keepalived memory dump summary for (%s) ]---\n", terminate_banner);
 	fprintf(log_op, "Total number of bytes not freed...: %d\n", sum);
 	fprintf(log_op, "Number of entries not freed.......: %d\n", n);
 	fprintf(log_op, "Maximum allocated entries.........: %d\n", number_alloc_list);
@@ -334,7 +339,7 @@ keepalived_free_final(char *banner)
 }
 
 void *
-keepalived_realloc(void *buffer, unsigned long size, char *file, char *function,
+keepalived_realloc(void *buffer, size_t size, char *file, char *function,
 		   int line)
 {
 	int i = 0;
@@ -393,7 +398,7 @@ keepalived_realloc(void *buffer, unsigned long size, char *file, char *function,
 	*(long *) ((char *) buf + size) = check;
 	alloc_list[i].csum = check;
 
-	fprintf(log_op, "realloc [%3d:%3d] %p, %4ld %s %d %s -> %p %4ld %s %d %s\n",
+	fprintf(log_op, "realloc [%3d:%3d] %p, %4zu %s %d %s -> %p %4zu %s %d %s\n",
 	       i, number_alloc_list, alloc_list[i].ptr,
 	       alloc_list[i].size, file, line, function, buf, size,
 	       alloc_list[i].file, alloc_list[i].line,
@@ -409,7 +414,7 @@ keepalived_realloc(void *buffer, unsigned long size, char *file, char *function,
 }
 
 void
-mem_log_init(const char* prog_name)
+mem_log_init(const char* prog_name, const char *banner)
 {
 	size_t log_name_len;
 	char *log_name;
@@ -451,5 +456,12 @@ mem_log_init(const char* prog_name)
 	}
 
 	free(log_name);
+
+	terminate_banner = banner;
+}
+
+void enable_mem_log_termination(void)
+{
+	atexit(keepalived_free_final);
 }
 #endif
