@@ -26,6 +26,7 @@
 
 /* system includes */
 #include <stdlib.h>
+#include <stdbool.h>
 
 /* Local defines */
 #ifdef _MEM_CHECK_
@@ -43,11 +44,14 @@
 extern size_t mem_allocated;
 
 /* Memory debug prototypes defs */
-extern void *keepalived_malloc(unsigned long, char *, char *, int);
+extern void *keepalived_malloc(size_t, char *, char *, int)
+		__attribute__((alloc_size(1))) __attribute__((malloc));
 extern int keepalived_free(void *, char *, char *, int);
-extern void *keepalived_realloc(void *, unsigned long, char *, char *, int);
-extern void keepalived_free_final(char *);
-extern void mem_log_init(const char *);
+extern void *keepalived_realloc(void *, size_t, char *, char *, int)
+		__attribute__((alloc_size(2)));
+
+extern void mem_log_init(const char *, const char *);
+extern void enable_mem_log_termination(void);
 
 #else
 
@@ -60,6 +64,5 @@ extern void *zalloc(unsigned long size);
 #endif
 
 /* Common defines */
-#define FREE_PTR(P) if((P)) FREE((P));
-
+#define FREE_PTR(p)	{ if (p) { FREE(p);} }
 #endif
