@@ -36,7 +36,7 @@
 #include "vector.h"
 
 /* Global definitions */
-#define CONF "/etc/keepalived/keepalived.conf"
+#define KEEPALIVED_CONFIG_FILE "/etc/keepalived/keepalived.conf"
 #define BOB  "{"
 #define EOB  "}"
 #define MAXBUF	1024
